@@ -69,12 +69,12 @@ class DefaultFdMixin(ProgressBarMixinBase):
         self.fd.write(line)
 
     def finish(self, *args, **kwargs):  # pragma: no cover
-        end = kwargs.pop('end', '\n')
-        ProgressBarMixinBase.finish(self, *args, **kwargs)
-
         if self._finished:
             return
 
+        end = kwargs.pop('end', '\n')
+        ProgressBarMixinBase.finish(self, *args, **kwargs)
+
         if end:
             self.fd.write(end)
 
@@ -178,6 +178,8 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         max_error (bool): When True the progressbar will raise an error if it
             goes beyond it's set max_value. Otherwise the max_value is simply
             raised when needed
+            prefix (str): Prefix the progressbar with the given string
+            suffix (str): Prefix the progressbar with the given string
 
     A common way of using it is like:
 
@@ -228,7 +230,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
     def __init__(self, min_value=0, max_value=None,
                  widgets=None, left_justify=True, initial_value=0,
                  poll_interval=None, widget_kwargs=None, custom_len=len,
-                 max_error=True, **kwargs):
+                 max_error=True, prefix=None, suffix=None, **kwargs):
         '''
         Initializes a progress bar with sane defaults
         '''
@@ -253,6 +255,8 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         self.max_value = max_value
         self.max_error = max_error
         self.widgets = widgets
+        self.prefix = prefix
+        self.suffix = suffix
         self.widget_kwargs = widget_kwargs or {}
         self.left_justify = left_justify
         self.value = initial_value
@@ -430,14 +434,13 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
 
     def __call__(self, iterable, max_value=None):
         'Use a ProgressBar to iterate through an iterable'
-        if max_value is None:
+        if max_value is not None:
+            self.max_value = max_value
+        elif self.max_value is None:
             try:
                 self.max_value = len(iterable)
             except TypeError:  # pragma: no cover
-                if self.max_value is None:
-                    self.max_value = base.UnknownLength
-        else:
-            self.max_value = max_value
+                self.max_value = base.UnknownLength
 
         self._iterable = iter(iterable)
         return self
@@ -525,6 +528,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             delta = timeit.default_timer() - self._last_update_timer
             poll_status = delta > self.poll_interval.total_seconds()
         else:
+            delta = 0
             poll_status = False
 
         # Do not update if value increment is not large enough to
@@ -538,7 +542,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             # ignore any division errors
             pass
 
-        return self.value > self.next_update or poll_status or self.end_time
+        return poll_status or self.end_time
 
     def update(self, value=None, force=False, **kwargs):
         'Updates the ProgressBar to a new value.'
@@ -550,7 +554,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
             if self.max_value is base.UnknownLength:
                 # Can't compare against unknown lengths so just update
                 pass
-            elif self.min_value <= value <= self.max_value:
+            elif self.min_value <= value <= self.max_value:  # pragma: no cover
                 # Correct value, let's accept
                 pass
             elif self.max_error:
@@ -626,6 +630,14 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         if self.widgets is None:
             self.widgets = self.default_widgets()
 
+        if self.prefix:
+            self.widgets.insert(0, widgets.FormatLabel(
+                self.prefix, new_style=True))
+
+        if self.suffix:
+            self.widgets.append(widgets.FormatLabel(
+                self.suffix, new_style=True))
+
         for widget in self.widgets:
             interval = getattr(widget, 'INTERVAL', None)
             if interval is not None:
@@ -635,7 +647,6 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
                 )
 
         self.num_intervals = max(100, self.term_width)
-        self.next_update = 0
 
         if self.max_value is not base.UnknownLength and self.max_value < 0:
             raise ValueError('Value out of range')
@@ -646,7 +657,7 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
 
         return self
 
-    def finish(self, end='\n'):
+    def finish(self, end='\n', dirty=False):
         '''
         Puts the ProgressBar bar in the finished state.
 
@@ -656,10 +667,13 @@ class ProgressBar(StdRedirectMixin, ResizableMixin, ProgressBarBase):
         Args:
             end (str): The string to end the progressbar with, defaults to a
                 newline
+            dirty (bool): When True the progressbar kept the current state and
+                won't be set to 100 percent
         '''
 
-        self.end_time = datetime.now()
-        self.update(self.max_value, force=True)
+        if not dirty:
+            self.end_time = datetime.now()
+            self.update(self.max_value, force=True)
 
         StdRedirectMixin.finish(self, end=end)
         ResizableMixin.finish(self)
